<?php

namespace EventPortal\Domain\Util\Sidebar;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Arr;
use Spatie\Permission\Exceptions\PermissionDoesNotExist;

class Sidebar
{
    /** @var \Illuminate\Support\Collection */
    protected $actions = [];

    /** @var string|null */
    protected $title;

    /** @var string|null */
    protected $status;

    /** @var Model */
    protected $model;

    /** @var array */
    private $context = [];

    /** @var bool */
    protected $hasBeenResolved = false;

    public function title(string $title): self
    {
        $this->title = $title;

        return $this;
    }

    public function status(?string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function model(Model $model): self
    {
        $this->model = $model;

        return $this;
    }

    public function addLinks(callable $callable): self
    {
        $this->actions[] = $callable;

        return $this;
    }

    /**
     * @param mixed ...$models
     */
    public function context(...$models): self
    {
        $keyedModels = [];

        foreach ($models as $model) {
            $keyedModels[$model::KEY] = $model;
        }

        $this->context = $keyedModels;

        if ($this->model) {
            $this->context = array_merge($this->context, [$this->model::KEY => $this->model]);
        }

        return $this;
    }

    public function render(): array
    {
        if ($this->hasBeenResolved === false) {
            $this->resolveLinks()
                ->filterPermittedLinks()
                ->renderLinks();
        }

        return [
            'title' => $this->title,
            'status' => $this->status,
            'children' => array_values($this->actions->toArray()),
        ];
    }

    private function resolveLinks(): self
    {
        $this->actions = collect($this->actions)
            ->transform(function ($action) {
                return app()->call($action, $this->getContext());
            })
            ->flatten(); // addLinks can be called multiple times

        $this->hasBeenResolved = true;

        return $this;
    }

    private function filterPermittedLinks(): self
    {
        $this->actions = $this
            ->actions
            ->filter(function ($link) {
                return $this->hasPermission($link->permission);
            })
            ->map(function (Link $link) {
                if ($link->children === null) {
                    return $link;
                }


                $link->children = array_filter($link->children, function ($link) {
                    return $this->hasPermission($link['permission']);
                });

                $link->children = array_values($link->children);

                return $link;
            });

        return $this;
    }

    private function renderLinks(): self
    {
        $this->actions = $this->actions->transform(function (Link $link) {
            return $link->render();
        });

        return $this;
    }

    private function hasPermission(?string $permission): bool
    {
        if ($permission === null) {
            return true;
        }

        if (Arr::has($this->context, 'user') === false) {
            return true;
        }

        try {
            return optional(Arr::get($this->context, 'user'))->admin->hasPermissionTo($permission);
        } catch (PermissionDoesNotExist $exception) {
            return false;
        }
    }

    private function getContext(): array
    {
        $parameters = $this->context;

        if ($this->model) {
            return array_merge($this->context, [$this->model::KEY => $this->model]);
        }

        return $parameters;
    }
}
