<?php

namespace EventPortal\Providers;

use EventPortal\Contracts\Event;
use EventPortal\Contracts\EventSpeaker;
use EventPortal\Domain\ActionList\Models\ActionList;
use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\Event as EventModel;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Event\Models\EventSpeaker as EventSpeakerModel;
use EventPortal\Domain\Util\Menu\Facades\Menu;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Exceptions\Handler;
use EventPortal\Http\Middleware\CanAccessAdmin;
use EventPortal\Http\Middleware\CheckPermission;
use EventPortal\Http\Middleware\CurrentTeam;
use EventPortal\Http\Middleware\Feature;
use EventPortal\Http\Middleware\IsDelegate;
use Illuminate\Contracts\Debug\ExceptionHandler;
use Illuminate\Routing\Router;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\ServiceProvider;
use Inertia\Inertia;

class AppServiceProvider extends ServiceProvider
{
    /** @var string */
    protected $root = __DIR__ . '/../..';

    public function boot(Router $router): void
    {
        $this->app->bind(Event::class, EventModel::class);
        $this->app->bind(EventSpeaker::class, EventSpeakerModel::class);

        $router->aliasMiddleware('current-team', CurrentTeam::class);
        $router->aliasMiddleware('check-permission', CheckPermission::class);
        $router->aliasMiddleware('can-access-admin', CanAccessAdmin::class);
        $router->aliasMiddleware('is-delegate', IsDelegate::class);
        $router->aliasMiddleware('feature', Feature::class);

        $this->app->booted(function (): void {
            Route::middleware(['web', 'auth', 'is-delegate'])
                ->namespace('EventPortal\Http\Controllers')
                ->group(function (): void {
                    $this->loadRoutesFrom("{$this->root}/routes/delegate.php");
                });

            Route::middleware(['web', 'auth', 'can-access-admin'])
                ->namespace('EventPortal\Http\Controllers\Admin')
                ->prefix('admin')
                ->group(function (): void {
                    $this->loadRoutesFrom("{$this->root}/routes/admin.php");
                });
        });

        $this->loadViewsFrom("{$this->root}/resources/views", 'EventPortal');

        $this->loadMigrationsFrom("{$this->root}/database/migrations");

        $this->loadFactoriesFrom("{$this->root}/database/factories");

        $this->app->bind(ExceptionHandler::class, Handler::class);

        $this->publishes([
            "{$this->root}/tables/delegate.php" => '/tables/delegates.php',
            "{$this->root}/tables/venue.php" => '/tables/venue.php',
            "{$this->root}/tables/event.php" => '/tables/event.php',
            "{$this->root}/tables/event-delegate.php" => '/tables/event-delegate.php',
            "{$this->root}/tables/line-manager-event.php" => '/tables/line-manager-event.php',
            "{$this->root}/tables/compliance-event.php" => '/tables/compliance-event.php',
        ], 'event-portal-tables');

        $this->publishes([
            "{$this->root}/forms/delegate.php" => '/forms/delegates.php',
            "{$this->root}/forms/event.php" => '/forms/event.php',
            "{$this->root}/forms/venue.php" => '/forms/venue.php',
            "{$this->root}/forms/event-delegate.php" => '/forms/event-delegate.php',
        ], 'event-portal-forms');

        $this->publishes([
            "{$this->root}/sidebars/event.php" => '/sidebars/event.php',
        ], 'event-portal-sidebars');

        $this->publishes([
            "{$this->root}/statistics/events.php" => '/statistics/events.php',
        ], 'event-portal-statistics');

        $this->publishes([
            "{$this->root}/quick-buttons/agenda.php" => '/quick-buttons/agenda.php',
        ], 'event-portal-quick-buttons');

        $this->publishes([
            "{$this->root}/notifications/admin.php" => '/notifications/admin.php',
            "{$this->root}/resources/views/email" => resource_path('views/vendor/eventportal/email'),
        ], 'event-portal-notifications');

        $this->publishes([
            "{$this->root}/action-lists/event.php" => '/action-lists/event.php',
        ], 'event-portal-action-lists');

        $this->publishes([
            "{$this->root}/config/permission.php" => config_path('/permission.php'),
            "{$this->root}/config/errors.php" => config_path('/errors.php'),
            "{$this->root}/config/state-machine.php" => config_path('/state-machine.php'),
            "{$this->root}/config/audit.php" => config_path('/audit.php'),
            "{$this->root}/config/page-headers.php" => config_path('/page-headers.php'),
            "{$this->root}/config/emails.php" => config_path('/emails.php'),
            "{$this->root}/config/roles.php" => config_path('/roles.php'),
            "{$this->root}/config/permissions.php" => config_path('/permissions.php'),
            "{$this->root}/config/dashboard.php" => config_path('/dashboard.php'),
            "{$this->root}/config/media-library.php" => config_path('/media-library.php'),
            "{$this->root}/config/venues.php" => config_path('/venues.php'),
        ], 'event-portal-config');

        $this->app->bind(ExceptionHandler::class, Handler::class);

        Inertia::setRootView('EventPortal::app');

        $this->setSharedData();
        Route::model('delegate', Delegate::class);
        Route::model('venue', Venue::class);
        Route::model('eventDelegate', EventDelegate::class);
        Route::model('admin', Admin::class);
        Route::model('eventSpeaker', EventSpeaker::class);
        Route::bind('form', static function ($idOrSlug) {
            return by_slug_or_id(Form::class, $idOrSlug);
        });
        Route::bind('event', static function ($value) {
            if (request()->is(['event/*'])) {
                return resolve(Event::class)
                    ->viewable()
                    ->whereSlug($value)
                    ->firstOrFail();
            }

            return resolve(Event::class)->whereId($value)->firstOrFail();
        });
        Route::model('actionList', ActionList::class);
    }

    private function setSharedData(): void
    {
        Inertia::share([
            'navigation' => function () {
                return $this->getNavigationData();
            },
            'errors' => function () {
                return session('errors') ? session('errors')->getBag('default')->getMessages() : (object) [];
            },
            'flash' => function () {
                return ['toast' => session('toast')];
            },
            'siteArea' => function () {
                return $this->viewingAdmin() ? 'admin' : 'delegate';
            },
            'footer' => [
                'links' => function () {
                    return $this->getFooterLinks();
                },
            ],
        ]);
    }

    private function isCompliance(): bool
    {
        return Auth::check() && Auth::user()->isAdmin() && Auth::user()->admin->isCompliance();
    }

    private function isADelegate(): bool
    {
        return Auth::check() && Auth::user()->isDelegate();
    }

    private function viewingAdmin(): bool
    {
        return Auth::check() && Auth::user()->isAdmin() && request()->is(['admin', 'admin/*']);
    }

    private function viewingDelegateDashboard(): bool
    {
        return request()->is(['/']);
    }

    private function viewingEvent(): bool
    {
        return request()->is(['event/*']);
    }

    private function viewingCancelledEvent(): bool
    {
        if ($this->viewingEvent() === false) {
            return false;
        }

        return request()->event->status === 'cancelled';
    }

    private function viewingAccount(): bool
    {
        return request()->is(['account/*']);
    }

    /**
     * Returns the data needed for the sites navigation.
     */
    private function getNavigationData(): ?array
    {
        if ($this->isCompliance()) {
            return Sidebar::make('compliance')->context(Auth::user())->render()['children'];
        }

        if ($this->viewingAdmin()) {
            return Sidebar::make('primary')->context(Auth::user())->render()['children'];
        }

        if ($this->isADelegate() && ($this->viewingDelegateDashboard() || $this->viewingAccount())) {
            return Menu::make('dashboard')
                ->user(Auth::user())
                ->get();
        }

        if ($this->isADelegate() && $this->viewingCancelledEvent()) {
            return Menu::make('event.cancelled')
                ->user(Auth::user())
                ->get();
        }

        if ($this->isADelegate() && $this->viewingEvent()) {
            return Menu::make('event')
                ->user(Auth::user())
                ->context(request()->event)
                ->get();
        }

        return null;
    }

    private function getFooterLinks(): array
    {
        return Menu::make('footer')->get();
    }
}
