<?php

namespace EventPortal\Tests\Unit\Table;

use Closure;
use EventPortal\Domain\Component\Column\TextColumn;
use EventPortal\Tests\Support\TestModel;
use EventPortal\Tests\TestCase;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class ColumnTest extends TestCase
{
    use RefreshDatabase;
    use MatchesSnapshots;

    /** @test **/
    public function name_is_set_from_label()
    {
        $column = TextColumn::make('Testing')->value('Chip Butty!');

        $this->assertEquals('testing', $column->name);
    }

    /** @test **/
    public function can_override_name_set_via_label()
    {
        $column = TextColumn::make('Testing')->name('chip');

        $this->assertEquals('chip', $column->name);
    }

    /** @test **/
    public function can_pass_in_string_value()
    {
        $column = TextColumn::make('Testing')->value('Chip Butty!');

        $this->assertEquals('Chip Butty!', $column->value);
    }

    /** @test **/
    public function can_pass_in_closure_value()
    {
        $column = TextColumn::make('Testing')->value(function () {
            return 'Chip Butty!';
        });

        $this->assertInstanceOf(Closure::class, $column->value);
    }

    /** @test **/
    public function can_set_as_default()
    {
        $column = TextColumn::make('Testing')->default();

        $this->assertTrue($column->isDefault());
    }

    /** @test **/
    public function can_set_as_searchable()
    {
        $column = TextColumn::make('Testing')->searchable();

        $this->assertTrue($column->isSearchable());
    }

    /** @test */
    public function set_tooltip()
    {
        $column = TextColumn::make('Testing')->tooltip(['This is your friendly tooltip']);

        $this->assertMatchesSnapshot($column->tooltip);
    }

    /** @test */
    public function set_tooltip_as_type_stats()
    {
        $column = TextColumn::make('Testing')
            ->statsTooltip(['primary' => 'Accepted', 'secondary' => 'Registered', 'tertiary' => 'Decline']);

        $this->assertMatchesSnapshot($column->tooltip);
    }

    /** @test **/
    public function can_mark_as_sortable(): void
    {
        $column = TextColumn::make('Testing')->sortable();

        $this->assertTrue($column->sortable);
    }

    /** @test **/
    public function can_give_sortable_a_callable(): void
    {
        $column = TextColumn::make('Testing')
            ->sortable(function (Builder $query) {
                return $query->orderBy('name', 'desc');
            });

        $this->assertTrue(is_callable($column->sortableCallable));
    }

    /** @test **/
    public function can_apply_sortable_callable_value(): void
    {
        $modalA = factory(TestModel::class)->create(['name' => 'A']);
        $modalB = factory(TestModel::class)->create(['name' => 'B']);

        $column = TextColumn::make('Testing')
            ->sortable(function (Builder $query) {
                return $query->orderBy('name', 'desc');
            });

        $query = tap(TestModel::query(), function ($query) use ($column): Builder {
            return app()->call($column->sortableCallable, ['query' => $query]);
        });

        $actual = $query->get();

        $this->assertSameModel($modalB, $actual->first());
        $this->assertSameModel($modalA, $actual->last());
    }

    /** @test **/
    public function can_pass_data_to_sortable_callable_value(): void
    {
        $modalA = factory(TestModel::class)->create(['name' => 'A']);
        $modalB = factory(TestModel::class)->create(['name' => 'B']);

        $column = TextColumn::make('Testing')
            ->sortable(function (Builder $query, string $direction) {
                return $query->orderBy('name', $direction);
            });

        $query = tap(TestModel::query(), function ($query) use ($column): Builder {
            return app()->call($column->sortableCallable, ['query' => $query, 'direction' => 'desc']);
        });

        $actual = $query->get();

        $this->assertSameModel($modalB, $actual->first());
        $this->assertSameModel($modalA, $actual->last());
    }
}
